program optcoeffs
use sep

implicit none
integer     ::n1,n2,i1,i2
integer     ::nv,niter,iv
real        ::d1,d2,v,dt
real	    ::b,c,vmin,vmax

double precision ::pi

double precision ::b1,b2,c1,c2
double precision ::dd1,dd2,ddt,vv

double precision ::ktransition,kmax

real, dimension(:), allocatable ::b1_out,b2_out,c1_out,c2_out


call sep_init()

pi=3.14159265358979323846

call from_history("n1",n1)
call from_history("n2",n2)

call from_history("d1",d1)
call from_history("d2",d2)

call from_par("nv",nv,2)
nv=max(2,nv)

call from_par("vmin",vmin,.333)
call from_par("vmax",vmax,1.)

call from_par("dt",dt,.3)

call from_par("b",b,4./3.)
call from_par("c",c,-1./12.)

call from_par("niter",niter,3)


call to_history("n1",nv)
call to_history("n2",4)
call to_history("n3",1)

call sep_close()

allocate(b1_out(nv))
allocate(b2_out(nv))
allocate(c1_out(nv))
allocate(c2_out(nv))


ktransition=.45*pi/dble(d1)
kmax=.6*pi/dble(d1)

dd1=dble(d1)
dd2=dble(d2)
ddt=dble(dt)

do iv=1,nv

  v=dble(iv-1)/dble(nv-1)*(vmax-vmin)+vmin
  vv=dble(v)

  b1=dble(b)*.75
  b2=dble(b)*.25
  c1=dble(c)*.75
  c2=dble(c)*.25

  call gaussnewton(vv,ddt,dd1,n1,niter,                  &
                   b1,c1,b2,c2,                                 &
                   ktransition,kmax)

  b1_out(iv)=b1
  b2_out(iv)=b2
  c1_out(iv)=c1
  c2_out(iv)=c2

if(nv < 10 )then
  write(0,*)' '
  write(0,*)'coeffs for v= ',v
  write(0,*)b1_out(iv),c1_out(iv),b2_out(iv),c2_out(iv)
end if

end do

call sep_write(b1_out)
call sep_write(b2_out)
call sep_write(c1_out)
call sep_write(c2_out)

deallocate (b1_out)
deallocate (b2_out)
deallocate (c1_out)
deallocate (c2_out)

call exit(0)

end program optcoeffs

include "gaussnewton_star4.f90"
include "objecfn_star4.f90"
include "lsqr.f90"
include "lsqrblas.f90"
include "aprod_cross4.f90"
