!
!
!  supply the spatial difference coefficients and the v and dt and d1 and d2 and I'll give you
!  the phase velocity spectrum in 2D as well as two error estimates, the first, the true error
!  the second the difference between the numerical vphase and the vphase used for design, which
!  is not necessarily a perfectly flat vphase (since that may be too hard to fit
!



program cross6_opt
use sep

implicit none

integer::n1,n2,i1,i2

real   ::d1,d2
real   ::b1,c1,e1
real   ::b2,c2,e2
real   ::br,cr,er,fr,gr
real   ::dr,fdt

real   ::vdtd1_squared,vdtd2_squared,vdtdr_squared
real   ::CC1_1,CC1_2,CC2_1,CC2_2,CC3_1,CC3_2
real   ::CC1_r,CC2_r,CC3_r,CC4_r,CC5_r

real   ::spacestuff,omega,preomega,k1,k2,kay,v,dt,courant,dtc
real   ::fspacestuff,fpreomega,fomega,kr
real   ::pi

real, dimension(:,:),allocatable::vphase,vtarget,error_a,error_f

call sep_init()

pi=3.1415926535

call from_history("n1",n1)
call from_history("n2",n2)

call from_history("d1",d1)
call from_history("d2",d2)

call from_par("v",v,1.)
call from_par("courant",courant,.3)

dtc=courant*min(d1,d2)/v

call from_par("dt",dt,dtc)

write(0,*)'computed dt ',dt

call from_par("b1",b1,4./3.)
call from_par("b2",b2,4./3.)

call from_par("c1",c1,-1./12.)
call from_par("c2",c2,-1./12.)

call from_par("e1",e1,0.)
call from_par("e2",e2,0.)

! just set the target to the 2nd order time 10th order space coeffs

br=5./3.
cr=-5./21.
er=5./126.
fr=-5./1008.
gr=1./3150.


call to_history("n3",4)

call sep_close()

allocate (vphase(n1,n2))
allocate (vtarget(n1,n2))
allocate (error_a(n1,n2))
allocate (error_f(n1,n2))


dr=min(d1,d2)
fdt=.1*dr/v



vdtd1_squared=v*v*dt*dt/(d1*d1)
vdtd2_squared=v*v*dt*dt/(d2*d2)
vdtdr_squared=v*v*fdt*fdt/(dr*dr)

do i2=1,n2
   k2=float(i2)*pi/(n2*d2)

   do i1=1,n1
      k1=float(i1)*pi/(n1*d1)

      kay=sqrt(k1*k1+k2*k2)

      kr=kay

      CC1_1=2.*cos(k1*d1)
      CC2_1=2.*cos(2.*k1*d1)
      CC3_1=2.*cos(3.*k1*d1)

      CC1_2=2.*cos(k2*d2)
      CC2_2=2.*cos(2.*k2*d2)
      CC3_2=2.*cos(3.*k2*d2)

      CC1_r=2.*cos(kr*dr)
      CC2_r=2.*cos(2.*kr*dr)
      CC3_r=2.*cos(3.*kr*dr)
      CC4_r=2.*cos(4.*kr*dr)
      CC5_r=2.*cos(5.*kr*dr)

      spacestuff= b1*vdtd1_squared*(CC1_1-2.)+                        &
                  b2*vdtd2_squared*(CC1_2-2.)+                        &
                  c1*vdtd1_squared*(CC2_1-2.)+                        &
                  c2*vdtd2_squared*(CC2_2-2.)+                        &
                  e1*vdtd1_squared*(CC3_1-2.)+                        &
                  e2*vdtd2_squared*(CC3_2-2.)

      preomega=spacestuff/2+1.
      omega=1./dt*acos(preomega)
      vphase(i1,i2)=omega/kay

      fspacestuff= br*vdtdr_squared*(CC1_r-2.)+                     &
                   cr*vdtdr_squared*(CC2_r-2.)+                     &
                   er*vdtdr_squared*(CC3_r-2.)+                     &
                   fr*vdtdr_squared*(CC4_r-2.)+                     &
                   gr*vdtdr_squared*(CC5_r-2.)

      fpreomega=fspacestuff/2+1.
      fomega=1./fdt*acos(fpreomega)
      vtarget(i1,i2)=fomega/kay



   end do
end do


! create the error maps


error_a=(vphase-v)/v
error_f=vphase-vtarget


call sep_write(vphase)
call sep_write(vtarget)
call sep_write(error_a)
call sep_write(error_f)

deallocate (vphase)
deallocate (vtarget)
deallocate (error_a)
deallocate (error_f)

call exit(0)

end program cross6_opt
