subroutine gaussnewton(v,dt,d1,d2,n1,n2,niter,b1,c1,e1,b2,c2,e2,     &
                       ktransition,kmax)

implicit none

integer            ::n1,n2,i1,i2
integer            ::niter,iter

double precision   ::ktransition,kmax
double precision   ::d1,d2,v,dt
double precision   ::b1,c1,e1,b2,c2,e2
double precision   ::b1_now,c1_now,b2_now,c2_now,e1_now,e2_now
double precision   ::b1_call,c1_call,b2_call,c2_call,e1_call,e2_call

double precision   ::db1,db2,dc1,dc2,de1,de2

double precision   ::testdiff

double precision   ::Err,dErr_db1,dErr_db2,dErr_dc1,dErr_dc2,dErr_de1,dErr_de2
double precision   ::Errp,dErr_db1p,dErr_db2p,dErr_dc1p,dErr_dc2p,dErr_de1p,dErr_de2p
double precision   ::Errm,dErr_db1m,dErr_db2m,dErr_dc1m,dErr_dc2m,dErr_de1m,dErr_de2m
double precision   ::pi

! LSQR variables

integer            ::itn,itnlim,nout,istop,iw
double precision   ::atol,btol,conlim,damp,anorm,acond,rnorm,arnorm,xnorm

double precision   ::u_lsqr(6),v_lsqr(6),w_lsqr(6),x_lsqr(6),se_lsqr(6)

double precision   ::matrix(6,6),rhs(6)


! set some fairly silly constants

db1=.0001
db2=.0001
dc1=.00001
dc2=.00001
de1=.000001
de2=.000001

!  set the current guess to whatever was passed in

b1_now=b1
b2_now=b2
c1_now=c1
c2_now=c2
e1_now=e1
e2_now=e2

do iter=1,niter

! first calculate the value of the objective function at the current guess

  b1_call=b1_now
  b2_call=b2_now
  c1_call=c1_now
  c2_call=c2_now
  e1_call=e1_now
  e2_call=e2_now

  call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
               Err, dErr_db1, dErr_db2, dErr_dc1, dErr_dc2, dErr_de1, dErr_de2,                   &
               ktransition,kmax                           )

write(0,*)'starting Error ',Err

! Set the right hand side, which is minus the gradient

    rhs(1)=-dErr_db1
    rhs(2)=-dErr_db2
    rhs(3)=-dErr_dc1
    rhs(4)=-dErr_dc2
    rhs(5)=-dErr_de1
    rhs(6)=-dErr_de2

! Now calculate the difference of the gradients to get the various second derivatives



! parameter b1, the first diff coeff on the 1 axis

    b1_call=b1_now+db1
    b2_call=b2_now
    c1_call=c1_now
    c2_call=c2_now
    e1_call=e1_now
    e2_call=e2_now

    call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
                 Errp, dErr_db1p, dErr_db2p, dErr_dc1p, dErr_dc2p, dErr_de1p, dErr_de2p,               &
                 ktransition,kmax                         )

    b1_call=b1_now-db1
    b2_call=b2_now
    c1_call=c1_now
    c2_call=c2_now
    e1_call=e1_now
    e2_call=e2_now

    call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
               Errm, dErr_db1m, dErr_db2m, dErr_dc1m, dErr_dc2m, dErr_de1m, dErr_de2m,                &
               ktransition,kmax                           )

!   Now complete this row of the matrix

    matrix(1,1)=(dErr_db1p-dErr_db1m)/(2.*db1)
    matrix(1,2)=(dErr_db2p-dErr_db2m)/(2.*db1)
    matrix(1,3)=(dErr_dc1p-dErr_dc1m)/(2.*db1)
    matrix(1,4)=(dErr_dc2p-dErr_dc2m)/(2.*db1)
    matrix(1,5)=(dErr_de1p-dErr_de1m)/(2.*db1)
    matrix(1,6)=(dErr_de2p-dErr_de2m)/(2.*db1)

! a little test

    testdiff=(Errp-Errm)/(2*db1)
    write(0,*)'test dErr_db1',testdiff,dErr_db1

! parameter b2, the first diff coeff on the 2 axis

   b1_call=b1_now
   b2_call=b2_now+db2
   c1_call=c1_now
   c2_call=c2_now
   e1_call=e1_now
   e2_call=e2_now

   call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
                Errp, dErr_db1p, dErr_db2p, dErr_dc1p, dErr_dc2p, dErr_de1p, dErr_de2p,              &
                ktransition,kmax                          )

   b1_call=b1_now
   b2_call=b2_now-db2
   c1_call=c1_now
   c2_call=c2_now
   e1_call=e1_now
   e2_call=e2_now

   call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
                Errm, dErr_db1m, dErr_db2m, dErr_dc1m, dErr_dc2m, dErr_de1m, dErr_de2m,              &
                ktransition,kmax                          )

!  Now complete this row of the matrix

   matrix(2,1)=(dErr_db1p-dErr_db1m)/(2.*db2)
   matrix(2,2)=(dErr_db2p-dErr_db2m)/(2.*db2)
   matrix(2,3)=(dErr_dc1p-dErr_dc1m)/(2.*db2)
   matrix(2,4)=(dErr_dc2p-dErr_dc2m)/(2.*db2)
   matrix(2,5)=(dErr_de1p-dErr_de1m)/(2.*db2)
   matrix(2,6)=(dErr_de2p-dErr_de2m)/(2.*db2)

! a little test

    testdiff=(Errp-Errm)/(2*db2)
    write(0,*)'test dErr_db2',testdiff,dErr_db2

! parameter c1, the second diff coeff on the 1 axis

    b1_call=b1_now
    b2_call=b2_now
    c1_call=c1_now+dc1
    c2_call=c2_now
    e1_call=e1_now
    e2_call=e2_now

    call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
                 Errp, dErr_db1p, dErr_db2p, dErr_dc1p, dErr_dc2p, dErr_de1p, dErr_de2p,              &
                 ktransition,kmax                         )

    b1_call=b1_now
    b2_call=b2_now
    c1_call=c1_now-dc1
    c2_call=c2_now
    e1_call=e1_now
    e2_call=e2_now

    call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
               Errm, dErr_db1m, dErr_db2m, dErr_dc1m, dErr_dc2m, dErr_de1m, dErr_de2m,              &
               ktransition,kmax                           )

!   Now complete this row of the matrix

    matrix(3,1)=(dErr_db1p-dErr_db1m)/(2.*dc1)
    matrix(3,2)=(dErr_db2p-dErr_db2m)/(2.*dc1)
    matrix(3,3)=(dErr_dc1p-dErr_dc1m)/(2.*dc1)
    matrix(3,4)=(dErr_dc2p-dErr_dc2m)/(2.*dc1)
    matrix(3,5)=(dErr_de1p-dErr_de1m)/(2.*dc1)
    matrix(3,6)=(dErr_de2p-dErr_de2m)/(2.*dc1)

! a little test

    testdiff=(Errp-Errm)/(2*dc1)
    write(0,*)'test dErr_dc1',testdiff,dErr_dc1

! parameter c2, the second diff coeff on the 2 axis

    b1_call=b1_now
    b2_call=b2_now
    c1_call=c1_now
    c2_call=c2_now+dc2
    e1_call=e1_now
    e2_call=e2_now

    call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
                 Errp, dErr_db1p, dErr_db2p, dErr_dc1p, dErr_dc2p, dErr_de1p, dErr_de2p,             &
                 ktransition,kmax                         )

    b1_call=b1_now
    b2_call=b2_now
    c1_call=c1_now
    c2_call=c2_now-dc2
    e1_call=e1_now
    e2_call=e2_now

    call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
               Errm, dErr_db1m, dErr_db2m, dErr_dc1m, dErr_dc2m, dErr_de1m, dErr_de2m,               &
               ktransition,kmax                           )

!   Now complete this row of the matrix

    matrix(4,1)=(dErr_db1p-dErr_db1m)/(2.*dc2)
    matrix(4,2)=(dErr_db2p-dErr_db2m)/(2.*dc2)
    matrix(4,3)=(dErr_dc1p-dErr_dc1m)/(2.*dc2)
    matrix(4,4)=(dErr_dc2p-dErr_dc2m)/(2.*dc2)
    matrix(4,5)=(dErr_de1p-dErr_de1m)/(2.*dc2)
    matrix(4,6)=(dErr_de2p-dErr_de2m)/(2.*dc2)

! a little test

    testdiff=(Errp-Errm)/(2*dc2)
    write(0,*)'test dErr_dc2',testdiff,dErr_dc2

! parameter e1, the third diff coeff on the 1 axis

    b1_call=b1_now
    b2_call=b2_now
    c1_call=c1_now
    c2_call=c2_now
    e1_call=e1_now+de1
    e2_call=e2_now

    call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
                 Errp, dErr_db1p, dErr_db2p, dErr_dc1p, dErr_dc2p, dErr_de1p, dErr_de2p,              &
                 ktransition,kmax                         )

    b1_call=b1_now
    b2_call=b2_now
    c1_call=c1_now
    c2_call=c2_now
    e1_call=e1_now-de1
    e2_call=e2_now

    call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
               Errm, dErr_db1m, dErr_db2m, dErr_dc1m, dErr_dc2m, dErr_de1m, dErr_de2m,              &
               ktransition,kmax                           )

!   Now complete this row of the matrix

    matrix(5,1)=(dErr_db1p-dErr_db1m)/(2.*de1)
    matrix(5,2)=(dErr_db2p-dErr_db2m)/(2.*de1)
    matrix(5,3)=(dErr_dc1p-dErr_dc1m)/(2.*de1)
    matrix(5,4)=(dErr_dc2p-dErr_dc2m)/(2.*de1)
    matrix(5,5)=(dErr_de1p-dErr_de1m)/(2.*de1)
    matrix(5,6)=(dErr_de2p-dErr_de2m)/(2.*de1)

! a little test

    testdiff=(Errp-Errm)/(2*de1)
    write(0,*)'test dErr_de1',testdiff,dErr_de1

! parameter e2, the third diff coeff on the 2 axis

    b1_call=b1_now
    b2_call=b2_now
    c1_call=c1_now
    c2_call=c2_now
    e1_call=e1_now
    e2_call=e2_now+de2

    call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
                 Errp, dErr_db1p, dErr_db2p, dErr_dc1p, dErr_dc2p, dErr_de1p, dErr_de2p,             &
                 ktransition,kmax                         )

    b1_call=b1_now
    b2_call=b2_now
    c1_call=c1_now
    c2_call=c2_now
    e1_call=e1_now
    e2_call=e2_now-de2

    call objecfn(v,dt,d1,d2,n1,n2,b1_call,b2_call,c1_call,c2_call,e1_call,e2_call,             &
               Errm, dErr_db1m, dErr_db2m, dErr_dc1m, dErr_dc2m, dErr_de1m, dErr_de2m,               &
               ktransition,kmax                           )

!   Now complete this row of the matrix

    matrix(6,1)=(dErr_db1p-dErr_db1m)/(2.*de2)
    matrix(6,2)=(dErr_db2p-dErr_db2m)/(2.*de2)
    matrix(6,3)=(dErr_dc1p-dErr_dc1m)/(2.*de2)
    matrix(6,4)=(dErr_dc2p-dErr_dc2m)/(2.*de2)
    matrix(6,5)=(dErr_de1p-dErr_de1m)/(2.*de2)
    matrix(6,6)=(dErr_de2p-dErr_de2m)/(2.*de2)

! a little test

    testdiff=(Errp-Errm)/(2*de2)
    write(0,*)'test dErr_de2',testdiff,dErr_de2




!  Solve the system of linear equations


!   write(0,*)matrix(1,1),matrix(1,2),matrix(1,3),matrix(1,4)
!   write(0,*)matrix(2,1),matrix(2,2),matrix(2,3),matrix(2,4)
!   write(0,*)matrix(3,1),matrix(3,2),matrix(3,3),matrix(3,4)
!   write(0,*)matrix(4,1),matrix(4,2),matrix(4,3),matrix(4,4)

   itnlim=12
   atol=0.
   btol=0.
   conlim=0.
   nout=22

   call lsqr(6,6,damp,.false.,1,iw,36,matrix,                         &
             rhs,v_lsqr,w_lsqr,x_lsqr,se_lsqr,                       &
             atol,btol,conlim,itnlim,nout,                           &
             istop,itn,anorm,acond,rnorm,arnorm,xnorm)

! Update the estimate of the coefficients

   b1_now=b1_now+x_lsqr(1)
   b2_now=b2_now+x_lsqr(2)
   c1_now=c1_now+x_lsqr(3)
   c2_now=c2_now+x_lsqr(4)
   e1_now=e1_now+x_lsqr(5)
   e2_now=e2_now+x_lsqr(6)

   write(0,*)x_lsqr(1),x_lsqr(3),x_lsqr(5)
   write(0,*)x_lsqr(2),x_lsqr(4),x_lsqr(6)


end do

b1=b1_now
b2=b2_now
c1=c1_now
c2=c2_now
e1=e1_now
e2=e2_now

return

end subroutine gaussnewton
