program optcoeffs_7pt_naive
use sep

implicit none
integer     ::n1,n2,i1,i2
integer     ::nv,niter,iv
real        ::d1,d2,d4,v,dt
real	    ::b,c,e,vmin,vmax

double precision ::pi

double precision ::b1,b2,c1,c2,e1,e2
double precision ::dd1,dd2,ddt,vv

double precision ::ktransition,kmax

real, dimension(:,:),allocatable ::vphase,error_a

real, dimension(:), allocatable ::b1_out,b2_out,c1_out,c2_out,e1_out,e2_out


call sep_init()

pi=3.14159265358979323846

call from_history("n1",n1)
call from_history("n2",n2)

call from_history("d1",d1)
call from_history("d2",d2)

call from_par("nv",nv,2)
nv=max(2,nv)

call from_par("vmin",vmin,.333)
call from_par("vmax",vmax,1.)

call from_par("dt",dt,.3)

call from_par("b",b,4./3.)
call from_par("c",c,-1./12.)
call from_par("e",e,0.)

call from_par("niter",niter,3)

call to_history("n3",2)
call to_history("n4",nv)


d4=(vmax-vmin)/float(nv-1)
call to_history("d3",1.)
call to_history("d4",d4)
call to_history("o4",vmin)


call to_history("n1",nv,"Ops")
call to_history("n2",6,"Ops")
call to_history("n3",1,"Ops")

call sep_close()

allocate(b1_out(nv))
allocate(b2_out(nv))
allocate(c1_out(nv))
allocate(c2_out(nv))
allocate(e1_out(nv))
allocate(e2_out(nv))

allocate(vphase(n1,n2))
allocate(error_a(n1,n2))


ktransition=.8*pi/dble(d1)
kmax=.90*pi/dble(d1)

dd1=dble(d1)
dd2=dble(d2)
ddt=dble(dt)

do iv=1,nv

  v=dble(iv-1)/dble(nv-1)*(vmax-vmin)+vmin
  vv=dble(v)

  b1=dble(b)
  b2=dble(b)
  c1=dble(c)
  c2=dble(c)
  e1=dble(e)
  e2=dble(e)

  call gaussnewton(vv,ddt,dd1,dd2,n1,n2,niter,                  &
                   b1,c1,e1,b2,c2,e2,                            &
                   ktransition,kmax)

  b1_out(iv)=b1
  b2_out(iv)=b2
  c1_out(iv)=c1
  c2_out(iv)=c2
  e1_out(iv)=e1
  e2_out(iv)=e2

  if(nv < 10 )then
    write(0,*)' '
    write(0,*)'coeffs for v= ',v
    write(0,*)b1_out(iv),c1_out(iv),e1_out(iv)
  end if

  call vphase_7pt(b1_out(iv),b2_out(iv),c1_out(iv),c2_out(iv),e1_out(iv),e2_out(iv),vphase,error_a,       &
                  n1,n2,d1,d2,dt,v)

  call sep_write(vphase)
  call sep_write(error_a)

end do

call sep_write(b1_out,"Ops")
call sep_write(b2_out,"Ops")
call sep_write(c1_out,"Ops")
call sep_write(c2_out,"Ops")
call sep_write(e1_out,"Ops")
call sep_write(e2_out,"Ops")

deallocate (b1_out)
deallocate (b2_out)
deallocate (c1_out)
deallocate (c2_out)
deallocate (e1_out)
deallocate (e2_out)
deallocate (vphase)
deallocate (error_a)

call exit(0)

end program optcoeffs_7pt_naive

include "gaussnewton_cross6.f90"
include "objecfn_bushes_cross6.f90"
include "lsqr.f90"
include "lsqrblas.f90"
include "aprod_cross6.f90"
include "vphase_7pt.f90"
